//============================================================================//
// Copyright 2014 Alachisoft.  All rights reserved.                    //
//============================================================================//

using System;
using System.IO;
using System.Data;
using Microsoft.Win32;
using System.Collections;
using System.Data.SqlClient;
using System.Globalization;
using DBSync.DataObject;

namespace DBSync.DataSource
{
        /// <summary>
        /// Class that helps read and write product information to SQL Server
        /// datasource.
        /// </summary>
        internal class SqlDatasource
        {
            private SqlConnection _connection;
            private string _connString = @"Server=localhost;initial catalog=Northwind;User ID = sa; password =;";

            /// <summary>
            /// Returns the connection string of this datasource.
            /// </summary>
            public string ConnString { get { return _connString; } }

            /// <summary>
            /// Establish connection with the datasource.
            /// </summary>
            /// <param name="connString"></param>
            public void Connect(string connString)
            {
                if (connString != "")
                    _connString = connString;
                _connection = new SqlConnection(_connString);
                try
                {
                    _connection.Open();
                }
                catch (Exception ex)
                {
                    System.Diagnostics.Debug.WriteLine(ex.Message);
                    throw ex;
                }
            }

            /// <summary>
            /// Releases the connection.
            /// </summary>
            public void DisConnect()
            {
                try
                {
                    if (_connection != null && _connection.State != ConnectionState.Closed)
                        _connection.Close();
                }
                catch (Exception)
                {
                    throw;
                }
            }

            /// <summary>
            /// Loads a <see cref="Product"/> object from the datasource.
            /// </summary>
            /// <param name="key"></param>
            /// <returns></returns>
            public object LoadProduct(Object key)
            {
                object[] productId = { key };
                SqlDataReader reader = null;
                try
                {
                    lock (this)
                    {
                        SqlCommand cmd = _connection.CreateCommand();
                        cmd.CommandText = String.Format(CultureInfo.InvariantCulture,
                                            "Select ProductID, ProductName, UnitPrice " +
                                            "From dbo.Products where ProductID = '{0}'",
                                            productId);
                        reader = cmd.ExecuteReader();
                        if (reader.Read())
                        {
                            Product objProduct = new Product();

                            objProduct.ProductID = Convert.ToInt32(reader["ProductID"]);
                            objProduct.ProductName = reader["ProductName"].ToString();
                            objProduct.UnitPrice = Convert.ToInt32(reader["UnitPrice"]);
                            reader.Close();
                            return objProduct;
                        }
                        reader.Close();
                    }
                }
                catch (Exception e)
                {
                    return e;
                }
                return null;
            }
            /// <summary>
            /// returns the top <paramref name="numOfRows"/> from the datasource
            /// </summary>
            /// <param name="number of rows"></param>
            /// <returns></returns>
            public Product[] LoadProducts(int numOfRows)
            {
                if (numOfRows <= 0)
                {
                    throw new Exception("invalid parameter value: numOfRows can not be less than 1");
                }

                SqlDataReader reader = null;
                try
                {
                    SqlCommand cmd = _connection.CreateCommand();
                    cmd.CommandText = String.Format(CultureInfo.InvariantCulture,
                                        "Select Top {0} ProductID, ProductName, UnitPrice " +
                                        "From dbo.Products",
                                        numOfRows);
                    reader = cmd.ExecuteReader();

                    Product[] objProducts = new Product[numOfRows];
                    for (int i = 0; i < numOfRows; i++)
                    {
                        if (reader.Read())
                        {
                            objProducts[i] = new Product();

                            objProducts[i].ProductID = Convert.ToInt32(reader["ProductID"]);
                            objProducts[i].ProductName = reader["ProductName"].ToString();
                            objProducts[i].UnitPrice = Convert.ToInt32(reader["UnitPrice"]);
                        }
                    }
                    reader.Close();
                    return objProducts;
                }
                catch (Exception e)
                {
                    System.Diagnostics.Trace.WriteLine(e);
                }

                return null;
            }
        }
    }
